package org.vigia.client.communications;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.HashSet;
import java.util.Set;

import org.vigia.shared.communications.Constants;
import org.vigia.shared.communications.ELog;
import org.vigia.shared.communications.LogListener;

/**
 * Implements the client control communication.
 * 
 * @author jrios
 */
public class ControlCommunicator {
	/**
	 * The socket to connect with the server.
	 */
	protected Socket socket;
	
	/**
	 * The registered listeners.
	 */
	protected Set<LogListener> logListeners;
	
	/**
	 * Connects to the remote camera data port.
	 * @param host The remote host.
	 * @param controlPort The remote port.
	 * @throws IOException Error with the IO.
	 * @throws UnknownHostException Host not found.
	 */
	public ControlCommunicator(String host, int controlPort) throws UnknownHostException, IOException {
		logListeners = new HashSet<LogListener>();
		socket = new Socket(host, controlPort);
		
		new ControlCommunicatorReader().start();
	}

	/**
	 * Adds a listener to obtain control messages from the remote camera.
	 * 
	 * @param logListener an implementation of the control listener.
	 */
	public void addLogListener(LogListener logListener) {
		logListeners.add(logListener);
	}
	
	/**
	 * Initializes the remote camera properly and starts the video transmission.
	 * 
	 * @param dataPort the client data port.
	 * @throws IOException Error sending the command.
	 */
	public void init(int dataPort) throws IOException {
		ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
		oos.writeInt(Constants.INIT);
		oos.writeInt(dataPort);
		oos.flush();
	}
	
	/**
	 * Send to the remote camera a move command in an absolute way.
	 * 
	 * @param pan pan position.
	 * @param tilt tilt position.
	 * @param zoom zoom position.
	 * @throws IOException Error sending the command.
	 */
	public void move(int pan, int tilt, int zoom) throws IOException {
		ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
		oos.writeInt(Constants.MOVE);
		oos.writeInt(pan);
		oos.writeInt(tilt);
		oos.writeInt(zoom);
		oos.flush();
	}
	
	/**
	 * Closes the control communicator.
	 * @throws IOException Error closing the socket.
	 */
	public void close() throws IOException {
		socket.close();
	}
	
	/**
	 * Check if the communications are closed.
	 * @return Is it's closed.
	 */
	public boolean isClosed() {
		return socket.isClosed();
	}
	
	/**
	 * This class reads from the socket and sends the commands to the listeners.
	 * @author jrios
	 */
	protected class ControlCommunicatorReader extends Thread {
		/**
		 * The default constructor.
		 */
		public ControlCommunicatorReader() {
			this.setDaemon(true);
			this.setName("ControlCommunicatorReader");
		}
		/**
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			while(!socket.isClosed() && socket.isConnected()){
				try {
					ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());
					switch(ois.readInt()) {
						case Constants.LOG:
							ELog type = (ELog)ois.readObject();
							String message = ois.readUTF();
							for(LogListener ll : logListeners)
								ll.onLog(type, message);
						break;
					}
				} catch (Exception e) {
					for(LogListener ll : logListeners)
						ll.onLog(ELog.Error, e.getMessage());
				}
			}
			for(LogListener ll : logListeners)
				ll.onLog(ELog.Warning, "Control socket connection lost.");
		}
	}
}
