package org.vigia.client.communications;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.util.HashSet;
import java.util.Set;

import javax.imageio.ImageIO;

import org.vigia.shared.communications.FrameListener;

/**
 * Implements the client data communication.
 * 
 * @author jrios
 */
public class DataCommunicator {
	/**
	 * The UDP socket.
	 */
	protected DatagramSocket socket;
	
	/**
	 * The registered listeners.
	 */
	protected Set<FrameListener> frameListeners;
	
	/**
	 * To end the socket.
	 */
	protected boolean end = false;
	
	/**
	 * Creates a data communicator on the port specified.
	 * @param dataPort The data port.
	 * @throws SocketException Error creating the socket.
	 */
	public DataCommunicator(int dataPort) throws SocketException {
		frameListeners = new HashSet<FrameListener>();
		socket = new DatagramSocket(dataPort);
		new DataCommunicatorReader().start();
	}

	/**
	 * Adds a listener to obtain the frames from the remote camera.
	 * 
	 * @param frameListener an implementation of the data listener.
	 */
	public void addFrameListener(FrameListener frameListener) {
		frameListeners.add(frameListener);
	}
	
	/**
	 * Closes the data communications.
	 */
	public void close() {
		end = true;
		socket.close();
	}
	
	/**
	 * This class reads from the socket and sends the commands to the listeners.
	 * @author jrios
	 */
	protected class DataCommunicatorReader extends Thread {
		/**
		 * The default size of the buffer.
		 */
		protected int BUFFER_SIZE = 65536;
		/**
		 * The buffer for read the UDP packages.
		 */
		protected DatagramPacket packet;
		
		/**
		 * The default constructor.
		 */
		public DataCommunicatorReader() {
			this.setDaemon(true);
			this.setName("DataCommunicatorReader");
			this.packet = new DatagramPacket(new byte[BUFFER_SIZE], BUFFER_SIZE);
		}
		
		/**
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			long seq=0;
			byte[] aux = new byte[14];
			
			while(!end){
				try {
					socket.receive(packet);
					
					ByteArrayInputStream bais = new ByteArrayInputStream(packet.getData());
					
					bais.read(aux);
					
					ByteArrayInputStream seqArray = new ByteArrayInputStream(aux);
					ObjectInputStream ois = new ObjectInputStream(seqArray);
					long nseq = ois.readLong();
					
					if(seq > nseq)
						continue;

					BufferedImage bi = ImageIO.read(bais);
					
					for(FrameListener fl : frameListeners)
						fl.onNewFrame(bi);

				} catch (IOException e) {}
			}
		}
	}
}
