package org.vigia.client.driver;

import java.util.ArrayList;
import java.util.List;

import org.vigia.client.exception.VigiaClientException;

import wiiusej.Wiimote;
import wiiusej.WiiUseApiManager;
import wiiusej.wiiusejevents.physicalevents.ExpansionEvent;
import wiiusej.wiiusejevents.physicalevents.IREvent;
import wiiusej.wiiusejevents.physicalevents.MotionSensingEvent;
import wiiusej.wiiusejevents.physicalevents.WiimoteButtonsEvent;
import wiiusej.wiiusejevents.utils.WiimoteListener;
import wiiusej.wiiusejevents.wiiuseapievents.ClassicControllerInsertedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.ClassicControllerRemovedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.DisconnectionEvent;
import wiiusej.wiiusejevents.wiiuseapievents.GuitarHeroInsertedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.GuitarHeroRemovedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.NunchukInsertedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.NunchukRemovedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.StatusEvent;

/**
 * Driver implementation based on the API WiiUseJ.
 * @author Ada
 * @version 1.1 (22/11/08)
 */
public class WiiUseJDriver implements Driver, WiimoteListener{

	/**
	 * A list containing all the infrared listeners for the Wiimote.
	 */
	private List<IRListener> listeners;
	
	/**
	 * Indicates the first event received after a led is missing.
	 */
	private boolean first2Leds;
	
	/**
	 * The last good coordinates.
	 */
	private Coordinates oldCoordinates;
	
	/**
	 * Bad coordinates received after a led is missing.
	 */
	private Coordinates badCoordinates;
	
	/**
	 * Detects the Wiimote and starts the thread that polls the events
	 * of the Wiimote.
	 * @throws VigiaClientException driver error.
	 */
	public WiiUseJDriver() throws VigiaClientException {
		listeners = new ArrayList<IRListener>();
		try {
			Wiimote[] wiimote = WiiUseApiManager.getWiimotes(1, true);
			wiimote[0].addWiiMoteEventListeners(this);
			wiimote[0].activateIRTRacking();
		} catch (Exception e) {
			//throw new VigiaClientException("Error while detecting Wiimote.", e);
		}
		first2Leds = false;
		oldCoordinates = new Coordinates(0,0,150);
		badCoordinates = null;
	}
	
	/**
	 * @see org.vigia.client.driver.Driver#addIRListener(org.vigia.client.driver.IRListener)
	 */
	public void addIRListener(IRListener l) throws VigiaClientException {
		try {
			listeners.add(l);
		} catch (Exception e) {
			throw new VigiaClientException("Couldn't add the listener.", e);
		}
	}
	
	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onIrEvent(wiiusej.wiiusejevents.physicalevents.IREvent)
	 */
	public void onIrEvent(IREvent irEv) throws VigiaClientException{
		int xOffset = 300; //Change this offset to calibrate the x coordinate.
		Coordinates c = new Coordinates(irEv.getX()-xOffset, irEv.getY(), irEv.getZ());
		try {
			if (irEv.getIRPoints().length >= 2 && !first2Leds){
				oldCoordinates = c;
				for (IRListener l : listeners){
					l.onIREvent(c, irEv.getXVRes(), irEv.getYVRes());
				}
			} else if (irEv.getIRPoints().length >= 2 && first2Leds){
				for (IRListener l : listeners){
					l.onIREvent(oldCoordinates, irEv.getXVRes(), irEv.getYVRes());
				}
				if (badCoordinates == null){
					badCoordinates = c;
				} else if (c.getX()!=badCoordinates.getX() && c.getY()!=badCoordinates.getY()){
					first2Leds = false;
					badCoordinates = null;
				}
			} else {
				first2Leds = true;
			}
		} catch (Exception e) {
			throw new VigiaClientException("Error on IR tracking.", e);
		}
	}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onButtonsEvent(wiiusej.wiiusejevents.physicalevents.WiimoteButtonsEvent)
	 */
	public void onButtonsEvent(WiimoteButtonsEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onClassicControllerInsertedEvent(wiiusej.wiiusejevents.wiiuseapievents.ClassicControllerInsertedEvent)
	 */
	public void onClassicControllerInsertedEvent(
			ClassicControllerInsertedEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onClassicControllerRemovedEvent(wiiusej.wiiusejevents.wiiuseapievents.ClassicControllerRemovedEvent)
	 */
	public void onClassicControllerRemovedEvent(ClassicControllerRemovedEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onDisconnectionEvent(wiiusej.wiiusejevents.wiiuseapievents.DisconnectionEvent)
	 */
	public void onDisconnectionEvent(DisconnectionEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onExpansionEvent(wiiusej.wiiusejevents.physicalevents.ExpansionEvent)
	 */
	public void onExpansionEvent(ExpansionEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onGuitarHeroInsertedEvent(wiiusej.wiiusejevents.wiiuseapievents.GuitarHeroInsertedEvent)
	 */
	public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onGuitarHeroRemovedEvent(wiiusej.wiiusejevents.wiiuseapievents.GuitarHeroRemovedEvent)
	 */
	public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onMotionSensingEvent(wiiusej.wiiusejevents.physicalevents.MotionSensingEvent)
	 */
	public void onMotionSensingEvent(MotionSensingEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onNunchukInsertedEvent(wiiusej.wiiusejevents.wiiuseapievents.NunchukInsertedEvent)
	 */
	public void onNunchukInsertedEvent(NunchukInsertedEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onNunchukRemovedEvent(wiiusej.wiiusejevents.wiiuseapievents.NunchukRemovedEvent)
	 */
	public void onNunchukRemovedEvent(NunchukRemovedEvent arg0) {}

	/**
	 * @see wiiusej.wiiusejevents.utils.WiimoteListener#onStatusEvent(wiiusej.wiiusejevents.wiiuseapievents.StatusEvent)
	 */
	public void onStatusEvent(StatusEvent arg0) {}

}
