package org.vigia.client.driver.util;

import org.vigia.client.driver.Coordinates;
import org.vigia.client.driver.PTZ;

/**
 * Transforms coordinates into PTZ and stores the position of the user
 * @author Ada
 */
public class MovementProcessor {
	
	/**
	 * The coordinates of the user.
	 */
	protected Coordinates c;
	/**
	 * True if no movement has been processed
	 */
	protected boolean init;
	
	/**
	 * Hysteresis for the x coordinate.
	 */
	protected final static int HYSTERESIS_X = 10;
	
	/**
	 * Hysteresis for the y coordinate.
	 */
	protected final static int HYSTERESIS_Y = 10;
	
	/**
	 * Hysteresis for the z coordinate.
	 */
	protected final static float HYSTERESIS_Z = 10;
	
	/**
	 * Multiplicative constant for the pan.
	 */
	protected final static double KPAN = 1.6;
	
	/**
	 * Multiplicative constant for the pan.
	 */
	protected final static double KTILT = 1.6;
	
	
	
	/**
	 * Constructor.
	 */
	public MovementProcessor(){
		c = new Coordinates(0,0,0);
		init = true;
	}
	
	//TODO: remove the parameters xVres and yVres if don't needed.
	/**
	 * Calculates the ptz movement.
	 * @param c the new coordinates of the user.
	 * @param xVRes IR virtual screen x resolution.
	 * @param yVRes IR virtual screen y resolution.
	 * @return the PTZ movement to be done by the remote camera, null if no movement needs to be done.
	 */
	public PTZ calculatePTZ(Coordinates c, int xVRes, int yVRes) {
		PTZ res = null;
		int pan = 0, tilt = 0, zoom = 100;
		int xC = c.getX();
		int yC = c.getY();
		float zC = c.getZ();
		boolean move = false;
		
		if (init) {
			this.c.setX(xC);
			this.c.setY(yC);
			this.c.setZ(zC);
			move = true;
			init = false;
		} else {
			if (movePan(xC)) {
				this.c.setX(xC);
				move = true;
			}
			if (moveTilt(yC)) {
				this.c.setY(yC);
				move = true;
			}
			if (moveZoom(zC)) {
				if (zC!=0){
					this.c.setZ(zC);
					move = true;
				}
			}
		}
		if (move) {
			move = false;
			pan = calculatePan(this.c.getX(), this.c.getY(), this.c.getZ(), xVRes);
			tilt = calculateTilt(this.c.getX(), this.c.getY(), this.c.getZ(), yVRes);
			zoom = calculateZoom(this.c.getZ());
			res = new PTZ(pan,tilt,zoom);
		}
		return res;
	}
	
	/**
	 * Calculates the pan movement.
	 * @param x the x coordinate.
	 * @param y the y coordinate.
	 * @param z the z coordinate.
	 * @param xVRes IR virtual screen x resolution.
	 * @return the calculated pan.
	 */
	protected int calculatePan(int x, int y, float z, int xVRes){
		double xD = (double) x;
		double yD = (double) y;
		double zD = (double) z;
		
		if (zD == 0) {
			zD = 800; //Average value of the distance from the virtual screen.
		}
		
		//Calculate the projected z by Pitagoras' theorem.
		double zX = Math.sqrt(Math.pow(zD, 2) - Math.pow(yD, 2));
		
		//Calculate the angle of the camera's turn.
		double pan = (Math.asin(xD / zX)*180)/Math.PI;
		
		//Calculate the sign (x and pan must be of equal sign).
		if (x<0 && pan>0){
			pan = -1*pan;
		} else if (x>0 && pan<0){
			pan = -1*pan;
		}
		
		pan = pan*KPAN;
		
		return (int) pan;
	}

	/**
	 * Calculates the tilt movement.
	 * @param x the x coordinate
	 * @param y the y coordinate.
	 * @param z the z coordinate.
	 * @param yVRes IR virtual screen y resolution.
	 * @return the calculated tilt.
	 */
	protected int calculateTilt(int x, int y, float z, int yVRes){
		double xD = (double) x;
		double yD = (double) y;
		double zD = (double) z;
		
		if (zD == 0) {
			zD = 800; //Average value of the distance from the virtual screen.
		}
		
		//Calculate the projected z by Pitagoras' theorem.
		double zY = Math.sqrt(Math.pow(zD, 2) - Math.pow(xD, 2));
		
		//Calculate the angle of the camera's turn.
		double tilt = (Math.asin(yD / zY)*180)/Math.PI;
		
		//Calculate the sign (y and tilt must be of equal sign).
		if (y<0 && tilt>0){
			tilt = -1*tilt;
		} else if (y>0 && tilt<0){
			tilt = -1*tilt;
		}
		
		tilt = tilt*KTILT;
		
		return (int) tilt;
	}
	
	/**
	 * Calculates the zoom movement.
	 * @param z the z coordinate.
	 * @return the calculated zoom.
	 */
	protected int calculateZoom(float z){
		//The maximum distance is 999.0, and the minimum is 0.
		
		//The more distance, the more zoom (if we look through a window
		//and get further of it, the objects we observe take a bigger 
		//proportion of the window)
		
		//If the distance grows up, the zoom will be bigger.
		
		double zD = (double) z;
		//If the distance is 625, the zoom is 100%.
		//If the distance is 999.0, the zoom is 200% (200%)
		//TODO constants
		double zoom = 200 - (((zD - 625.0)/(999.0 - 625.0))*100.0);
		return (int) zoom;
	}
	
	/**
	 * Decides if a pan movement needs to be done.
	 * @param x the x coordinate.
	 * @return true if a pan movement needs to be done.
	 */
	protected boolean movePan(int x){
		return Math.abs(c.getX() - x) > HYSTERESIS_X;
	}
	
	/**
	 * Decides if a tilt movement needs to be done.
	 * @param y the y coordinate.
	 * @return true if a tilt movement needs to be done.
	 */
	protected boolean moveTilt(int y){
		return Math.abs(c.getY() - y) > HYSTERESIS_Y;
	}
	
	/**
	 * Decides if a zoom movement needs to be done.
	 * @param z the z coordinate.
	 * @return true if a zoom movement needs to be done.
	 */
	protected boolean moveZoom(float z){
		return Math.abs(c.getZ() - z) > HYSTERESIS_Z;
	}
}
