package org.vigia.client.gui.controller;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.imageio.ImageIO;
import javax.swing.JOptionPane;

import org.vigia.client.communications.Communicator;
import org.vigia.client.communications.RemoteCameraCommunicator;
import org.vigia.client.driver.Coordinates;
import org.vigia.client.driver.Driver;
import org.vigia.client.driver.PTZ;
import org.vigia.client.driver.util.MovementProcessor;
import org.vigia.client.gui.view.View;
import org.vigia.shared.communications.ELog;

/**
 * An implementation of the graphic user interface's controller.
 * @author Ada & Daniel Hector Stolfi Rosso & jrios & Toshi
 *
 */
public class ClientController implements Controller {
	
	/**
	 * The name of the log file.
	 */
	protected static final String LOG_FILE = "vigia.log";
	/**
	 * The view.
	 */
	protected View view;
	/**
	 * The driver of the IR sensor.
	 */
	protected Driver driver;
	/**
	 * The communicator.
	 */
	protected Communicator comm;
	/**
	 * The movement processor.
	 */
	protected MovementProcessor mp;
	
	/**
	 * Creates a new controler for the view and the model.
	 * @param view The graphic user interface's view.
	 * @param driver The driver of the IR sensor.
	 * @param mp The movement processor.
	 */
	public ClientController(View view, Driver driver, MovementProcessor mp) {
		this.view = view;
		this.view.controller(this);
		view.setStatusInfo("Desconectado");
		this.driver = driver;
		this.driver.addIRListener(this);
		this.mp = mp;
	}

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equalsIgnoreCase(View.ActCmdPANEL_CONN)){
			view.showConnectToServerPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdPANEL_DISC)){
			try {
				if ((comm!=null) && (!comm.isClosed())) {
					// Close if it was connected
					comm.close();
				}
			} catch (Exception e2) {
			}
			view.setStatusInfo("Desconectado");
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdPANEL_CLOSE)) {
			System.exit(0);
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdPANEL_CONFIG)) {
			view.showAdvancedConnectionSettingsPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdPANEL_SCRSHOT)) {
			view.showGetScreenShotPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdPANEL_SHOWLOG)) {
			view.showLog();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdPANEL_ABOUT)) {
			view.showAboutInfo();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdCONN_OK)) {
			view.closeConnectToServerPanel();
			try {
				if ((comm!=null) && (!comm.isClosed())) {
					// Close before Reconnect
					comm.close();
				}
			} catch (Exception e2) {
			}
			try {
				comm = new RemoteCameraCommunicator(view.getIPServer(),Integer.parseInt(view.getControlPort()),Integer.parseInt((view.getDataPort())));
				comm.addFrameListener(this);
				comm.addLogListener(this);
				view.setStatusInfo("Conectado: "+view.getIPServer()+":"+view.getDataPort()+":"+view.getControlPort());
			} catch (Exception e1) {
				JOptionPane.showMessageDialog((Component) view,
											  "No se puede conectar con el servidor",
											  "ERROR",
											  JOptionPane.ERROR_MESSAGE);
				
				view.setStatusInfo("Desconectado");
			}			
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdCONN_CANCEL)) {
			view.closeConnectToServerPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdCONN_ADVOPC)) {
			view.showAdvancedConnectionSettingsPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdADVOPC_OK)) {
			// Must disc and connect again to update parameters			
			view.closeAdvancedConnectionSettingsPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdADVOPC_CANCEL)) {
			view.closeAdvancedConnectionSettingsPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdSCRSHOT_CANCEL)) {
			view.closeGetScreenShotPanel();
		} else if(e.getActionCommand().equalsIgnoreCase(View.ActCmdSCRSHOT_OK)) {
			view.closeGetScreenShotPanel();
			try {
				ImageIO.write(view.getImage(),"jpg",new File(view.getScreenShotURL()+"/"+ new SimpleDateFormat("yyyyMMddHHmmss").format(new Date())+".jpg"));
			} catch (IOException e2) {
				JOptionPane.showMessageDialog((Component) view,
						  					  "No se puede guardar la instant�nea",
						  					  "ERROR",
						  					  JOptionPane.ERROR_MESSAGE);
			}						
		}
	}

	/**
	 * @see org.vigia.shared.communications.FrameListener#onNewFrame(java.awt.image.BufferedImage)
	 */
	public void onNewFrame(BufferedImage frame) {
		view.showImage(frame);
	}

	/**
	 * @see org.vigia.client.driver.IRListener#onIREvent(org.vigia.client.driver.Coordinates, int, int)
	 */
	public void onIREvent(Coordinates c, int xVRes, int yVRes) {
		PTZ ptz = mp.calculatePTZ(c, xVRes, yVRes);
		try {
			comm.move(ptz.getPan(), ptz.getTilt(), ptz.getZoom());
		} catch (Exception e) {
			//ignore
		}
	}
	
	/**
	 * @see org.vigia.shared.communications.LogListener#onLog(org.vigia.shared.communications.ELog, java.lang.String)
	 */
	public void onLog(ELog log, String message){
		view.appendLog(log, message);
		try {
	        BufferedWriter out = new BufferedWriter(new FileWriter(LOG_FILE,true));
	        out.write("["+new SimpleDateFormat("MMM dd H:mm:ss").format(new Date())+"] ");
	        out.write(log.name() + " - "+ message + '\n');   
	        out.close();
		} catch (IOException e) {
			// TODO: handle exception
		}
	}
}
