package org.vigia.server.communications;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashSet;
import java.util.Set;

import org.vigia.server.communications.CommandListener;
import org.vigia.server.exception.NotConnectedException;
import org.vigia.shared.communications.Constants;
import org.vigia.shared.communications.ELog;

/**
 * Implements the server control communication.
 * 
 * @author jrios
 */
public class ControlCommunicator {
	/**
	 * The socket to wait clients.
	 */
	protected ServerSocket server;
	/**
	 * The socket to communicate with the clients.
	 */
	protected Socket client;
	/**
	 * The registered listeners.
	 */
	protected Set<CommandListener> commandListeners;
	
	/**
	 * Creates a new ControlCommunicator.
	 * 
	 * @param controlPort The control port.
	 * @throws IOException Error creating the socket.
	 */
	public ControlCommunicator(int controlPort) throws IOException {
		commandListeners = new HashSet<CommandListener>();
		
		server = new ServerSocket(controlPort);
		
		new ControlCommunicatorReader().start();
	}

	/**
	 * Adds a listener to obtain control messages from the remote camera.
	 * 
	 * @param commandListener an implementation of the control listener.
	 */
	public void addCommandListener(CommandListener commandListener) {
		commandListeners.add(commandListener);
	}

	/**
	 * Sends a log message to the client.
	 * 
	 * @param log the kind of log event.
	 * @param message the log message.
	 * @throws NotConnectedException There isn't any client.
	 * @throws IOException Error writing to the socket.
	 */
	public void sendLog(ELog log, String message) throws NotConnectedException, IOException {
		if(client==null) throw new NotConnectedException("There isn't a client yet.");
		ObjectOutputStream oos = new ObjectOutputStream(client.getOutputStream());
		oos.writeInt(Constants.LOG);
		oos.writeObject(log);
		oos.writeUTF(message);
		oos.flush();
	}
	
	/**
	 * This class accepts clients, reads from the socket and sends the commands to the listeners.
	 * @author jrios
	 */
	protected class ControlCommunicatorReader extends Thread {
		/**
		 * The default constructor.
		 */
		public ControlCommunicatorReader() {
			this.setName("ControlCommunicatorReader");
		}
		
		/**
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			while(true) {
				try {
					client = server.accept();
					System.out.println("User connected.");
					while(client.isConnected()) {
							ObjectInputStream ois = new ObjectInputStream(client.getInputStream());
							switch(ois.readInt()) {
								case Constants.MOVE:
									int pan = ois.readInt();
									int tilt = ois.readInt();
									int zoom = ois.readInt();
									
									for(CommandListener cl : commandListeners)
										cl.onMove(pan, tilt, zoom);
								break;
								case Constants.INIT:
									int clientPort = ois.readInt();
									
									for(CommandListener cl : commandListeners)
										cl.onInit(clientPort);
								break;
							}
					}
				} catch (IOException e) {
					System.out.println("User disconnected.");
				}
				for(CommandListener cl : commandListeners)
					cl.onClose();
			}
		}
	}
}
