package org.vigia.server.driver.camera;

import java.awt.image.BufferedImage;
import java.util.HashSet;
import java.util.Set;

import org.vigia.shared.communications.ELog;
import org.vigia.shared.communications.FrameListener;
import org.vigia.shared.communications.LogListener;
import org.vigia.server.exception.*;
import org.vigia.server.driver.util.*;

import sun.net.util.IPAddressUtil;


/**
 * The camera driver class
 * @author jesus
 * 
 */
public class CameraDriver implements Driver {
	/**
	 * List of camera's frame listeners
	 */
	protected Set<FrameListener> frameListeners;
	
	/**
	 * List of camera's log listeners
	 */
	protected Set<LogListener> logListeners;
	
	/**
	 * Server's IP address
	 */
	private String m_Server;
	/**
	 * Server's port
	 */
	private int m_Port;
	
	/**
	 * Current XPos of the camera
	 */
	private int currentXPos;
	/**
	 * Current YPos of the camera
	 */
	private int currentYPos;
	/**
	 * Current ZPos of the camera
	 */
	private int currentZPos;
	
	/**
	 * Delay between frames
	 */
	private static final int DELAY = 40;  // ms  (25 FPS; could be 25-35ms)
	/**
	 * If true the camera's thread will stop
	 */
	private boolean stop = true;
	/**
	 * JMFCapture object (webcam)
	 */
	private JMFCapture camera = new JMFCapture();
	/**
	 * count of images (frames)
	 */
	private int imageCount = 0;
	/**
	 * total time of capturing period
	 */
	private long totalTime = 0;
	/**
	 * current frame
	 */
	private BufferedImage frame = null;

	/**
	 * The tread that grabs the frames
	 */
	protected FrameThread fThread;
	
	/**
	 * C constant, percentage of original image removed
	 */
	static private double C = 0.36;
	/**
	 * Kstep constant, size of step for camera's moves 
	 */
	static private int Kstep = 1;
	/**
	 * used for synchronize current x,y,z
	 */
	private Object XYZSync = new Object();
	
	
	/**
	 * CameraDriver class constructor
	 */
	public CameraDriver() {
		frameListeners = new HashSet<FrameListener>();
		logListeners = new HashSet<LogListener>();
		
		this.m_Server = "127.0.0.1";
        this.m_Port = 8090;
		
		currentXPos=0;
		currentYPos=0;
		currentZPos=150;
		
		goHome();
		fThread = new FrameThread();
		fThread.start();
	}
	
	/**
	 * CameraDriver class constructor
	 * @param server server IP address
	 * @param port server HTTP port 
	 * @throws CameraDriverException exception creating the camera driver
	 */
	public CameraDriver(String server, int port) throws CameraDriverException {
		
		if (!(IPAddressUtil.isIPv4LiteralAddress(server)) || (port < 1)){
			throw new CameraDriverException("Error creating camera driver: bad server ip or bad server port");
		}
		
		frameListeners = new HashSet<FrameListener>();
		logListeners = new HashSet<LogListener>();
		
		this.m_Server = server;
        this.m_Port = port;
       
		currentXPos=0;
		currentYPos=0;
		currentZPos=150;
		
		goHome();
		fThread = new FrameThread();
		fThread.start();
	}
	
	/**
	 * @see org.vigia.server.driver.camera.Driver#addFrameListener(org.vigia.shared.communications.FrameListener)
	 */
	public void addFrameListener(FrameListener frameListener) {
		frameListeners.add(frameListener);
	}

	/**
	 * @see org.vigia.server.driver.camera.Driver#addLogListener(org.vigia.shared.communications.LogListener)
	 */
	public void addLogListener(LogListener logListener) {
		logListeners.add(logListener);
	}

	/**
	 * Move to relative position (pan,tilt) and zoom
	 * @param pan pan movement
	 * @param tilt tilt movement
	 * @param zoom zoom movement
	 */
	public void move(int pan, int tilt, int zoom) {
		HttpClient httpclient = new HttpClient(m_Server, m_Port);
        String s1 = "";
        int xpos=0,ypos=0;
        String [] temp=null; 
        int cXPos=0,cYPos=0,cZPos=0;
        
        xpos = pan - currentXPos;
        ypos = tilt - currentYPos;
        
        if (zoom >= 100 && zoom <= 400){
        	cZPos = zoom;
        }
        else{
        	cZPos = currentZPos;
        }
        
        s1 = (new StringBuilder()).append("XPos=").append(xpos).append("&YPos=").append(ypos).append("&ZPos=").append(zoom).append("&UserID=console&RequestType=move&StepSize=").append(Kstep).append("&WhereIsTheFunctions=unknown").toString();
        
        String s2 = httpclient.REQUEST("/RemoteProcessN.trackercamcontrol", s1, true);
        
        //Show POST Command Response
        //System.out.println("Return value: "+s2);
        
        temp = s2.split("'");
        if (s2.contains("TRUE")){
           	cXPos = Integer.parseInt(temp[5]);
           	cYPos = Integer.parseInt(temp[7]);
        }
            
        synchronized (XYZSync){
        	currentXPos=cXPos;
        	currentYPos=cYPos;
        	currentZPos=cZPos;
        }
        
        //Show Current X and Y position of PTZ base
        //System.out.println("currentXPos:"+currentXPos+" CurrentYPos:"+currentYPos);
        
		for(LogListener ll : logListeners)
		{
			if (s2.contains("ILLEGAL")){
				temp = s2.split("'");
				if (temp.length >= 9){
					ll.onLog(ELog.Error, temp[9]);
				}
			
			}else if (s2.contains("failed") || s2.contains("NOTRACKERPOD")){
				ll.onLog(ELog.Error, "Cannot connect with the server of PTZ base.");
			
			}else if (s2.contains("Warning")){
				ll.onLog(ELog.Warning, s2);
			
			}else if (s2.contains("TRUE")){
				//ll.onLog(ELog.Info, "XPos="+currentXPos+",YPos="+currentYPos+",ZPos="+currentZPos);
					
			}else{
				ll.onLog(ELog.Info, s2);
			}
			
		}
	}

	/**
	 * Go to home position
	 */
	public void goHome() {
		HttpClient httpclient = new HttpClient(m_Server, m_Port);
        String s1 = "";
        String [] temp=null;
        int cXPos=0,cYPos=0,cZPos=0;
        
        s1 = (new StringBuilder()).append("XPos=H").append("&YPos=H").append("&UserID=console&RequestType=move&StepSize=").append(Kstep).append("&WhereIsTheFunctions=unknown").toString();
        
        String s2 = httpclient.REQUEST("/RemoteProcessN.trackercamcontrol", s1, true);
        
        //Show POST Command Response
        //System.out.println("Return value: "+s2);
        
        temp = s2.split("'");
    	if (s2.contains("TRUE")){
    		cXPos = Integer.parseInt(temp[5]);
    		cYPos = Integer.parseInt(temp[7]);
    	}
    	cZPos = 150;
    	
        synchronized (XYZSync){
        	currentXPos=cXPos;
        	currentYPos=cYPos;
        	currentZPos=cZPos;
        }
        	
        //Show Current X and Y position of PTZ base
        //System.out.println("currentXPos:"+currentXPos+" CurrentYPos:"+currentYPos);
        
		for(LogListener ll : logListeners)
		{
			if (s2.contains("ILLEGAL")){
				temp = s2.split("'");
				if (temp.length >= 9){
					ll.onLog(ELog.Error, temp[9]);
				}
			
			}else if (s2.contains("failed") || s2.contains("NOTRACKERPOD")){
				ll.onLog(ELog.Error, "Cannot connect with the server of PTZ base.");
			
			}else if (s2.contains("Warning")){
				ll.onLog(ELog.Warning, s2);
			
			}else if (s2.contains("TRUE")){
				//ll.onLog(ELog.Info, "XPos="+currentXPos+",YPos="+currentYPos+",ZPos="+currentZPos);
			
			}else{
				ll.onLog(ELog.Info, s2);
			}
			
		}
	}
	
	/**
	 * Thread to get the frames from the camera
	 */
	protected class FrameThread extends Thread {
		/**
		 */
		public FrameThread(){
			this.setDaemon(true);
		}
		
		/**
		 * Main function of the Frame Thread
		 * @see java.lang.Thread#run()
		 */
		public void run() {
			long duration;
			BufferedImage im = null,tmp=null;
			int x=0,y=0,wp=0,hp=0,xp=0,yp=0,wpp=0,hpp=0;
			int cXPos=0,cYPos=0,cZPos=0;
			
			while (true) {
				synchronized (this) {
					while(stop){
						try {
							this.wait();
						} catch (InterruptedException e) {}
					}
				}
				
				long startTime = System.currentTimeMillis();
				im = camera.grabImage();  // take a snap
				duration = System.currentTimeMillis() - startTime;

				if (im == null)
					System.out.println("Problem loading image.");
				else {
					tmp = im;   // only update image if im contains something
					
					//------image processing-------
					int w = im.getWidth();
					int h = im.getHeight();
					
					synchronized (XYZSync){
						cXPos=currentXPos;
						cYPos=currentYPos;
						cZPos=currentZPos;
					}
					
					//crop the image
					x =(int) (((30+cXPos)/60.0)*w*C);
					y =(int) (((30+cYPos)/60.0)*h*C);
					wp = (int) (w*(1-C));
					hp = (int) (h*(1-C));
					tmp = ImageUtil.crop(tmp,x,y,wp,hp);
					
					xp = (int) ((1-(100.0/cZPos))*(wp/2.0));
					yp = (int) ((1-(100.0/cZPos))*(hp/2.0));
					wpp = (int) ((100.0/cZPos)*wp);
					hpp = (int) ((100.0/cZPos)*hp);
					tmp = ImageUtil.crop(tmp, xp, yp, wpp, hpp);
					
					//scale the image
					frame = ImageUtil.scale(tmp, w, h);
					
					//-----------------------------
					
					imageCount++;
					totalTime += duration;
					
					for(FrameListener fl : frameListeners)
						fl.onNewFrame(frame);
			   }

			   if (duration < DELAY) {
			     try {
			       Thread.sleep(DELAY-duration);  // wait until DELAY time has passed
			     } 
			     catch (Exception ex) {}
			   }
			 }

			 //camera.close();    // close down the camera
		}
	}

	/**
	 * Start to capture frames from the camera
	 * @see org.vigia.server.driver.camera.Driver#start()
	 */
	public void start() {
		synchronized (fThread) {
			stop=false;
			fThread.notifyAll();
		}
		goHome();
	}

	/**
	 * Stop to capture frames from the camera
	 * @see org.vigia.server.driver.camera.Driver#stop()
	 */
	public void stop() {
		stop = true;
	}
}

