package org.vigia.server.driver.camera;

import java.awt.*;
import java.awt.image.*;

import javax.media.*;
import javax.media.control.*;
import javax.media.format.*;
import javax.media.util.*;


/**
 * @author Jesus
 *
 */
public class JMFCapture implements ControllerListener
{
	
	/**
	 * Common name in WinXP/Vista
	 */
	private static final String CAP_DEVICE = "vfw:Microsoft WDM Image Capture (Win32):0";
	/**
	 * tries used while waiting for the BufferToImage object to be initialized
	 */
	private static final int MAX_TRIES = 7;
	/**
	 * period used while waiting for the BufferToImage object to be initialized
	 */
	private static final int TRY_PERIOD = 1000;   // ms
	
	/**
	 * Player object
	 */
	private Player p;
	/**
	 * FrameGrabbingControl object
	 */
	private FrameGrabbingControl fg;
	/**
	 * BufferToImage object
	 */
	private BufferToImage bufferToImage = null;
	/**
	 * closed device indication
	 */
	private boolean closedDevice;
	/**
	 * used for waiting until the player has started
	 */
	private Object waitSync = new Object();
	/**
	 * used for waiting until the player has started
	 */
	private boolean stateTransitionOK = true;


	/**
	 * JMFCapture constructor
	 */
	public JMFCapture(){
		closedDevice = true;   // since device is not available yet
		
		// link player to capture device
		try {
			//System.out.println(CaptureDeviceManager.getDeviceList(null));
			CaptureDeviceInfo cdi = CaptureDeviceManager.getDevice(CAP_DEVICE);
		 	p = Manager.createRealizedPlayer(cdi.getLocator());
		 	System.out.println("Created player");
		}
		catch (Exception e) {
			System.out.println("Failed to create player");
			System.exit(0);
		}
		
		p.addControllerListener(this);
		
		// create the frame grabber
		fg =  (FrameGrabbingControl) p.getControl("javax.media.control.FrameGrabbingControl");
		if (fg == null) {
			System.out.println("Frame grabber could not be created");
			System.exit(0);
		}
		
		// Change to 640x480 format
		FormatControl fc = (FormatControl) p.getControl("javax.media.control.FormatControl");
		Format[] formats = fc.getSupportedFormats();
		for(int i=0;i<formats.length;i++){
			System.out.println(formats[i]);
		}
		fc.setFormat(formats[0]);
		
		// wait until the player has started
		System.out.println("Starting the player...");
		p.start();
		if (!waitForStart()) {
			System.err.println("Failed to start the player.");
			System.exit(0);
		}
		
		waitForBufferToImage();
	}  

	
	/**
	 * wait for the player to enter its Started state
	 * @return state
	 */
	private boolean waitForStart(){ 
		synchronized (waitSync) {
		 try {
		   while (p.getState() != Controller.Started && stateTransitionOK)
			   waitSync.wait();
		 }
		 catch (Exception e) {}
		}
		return stateTransitionOK;
	} 


	/**
	 * Wait for the BufferToImage object to be initialized.
	 * May take several seconds to initialize this object, 
	 * so this method makes up to MAX_TRIES attempts
	 */
	private void waitForBufferToImage(){
		int tryCount = MAX_TRIES;
		System.out.println("Initializing BufferToImage...");
		while (tryCount > 0) 
		{
			if (hasBufferToImage())   
			  break;
			 
			try {   
			  System.out.println("Waiting...");
			  Thread.sleep(TRY_PERIOD);
			}
			catch (InterruptedException e)
			{  System.out.println(e);  }
			 
			tryCount--;
		}
		
		if (tryCount == 0) {
			System.out.println("Giving Up");
			System.exit(0);
		}
		
		closedDevice = false;
	}


	/**
	 *  The BufferToImage object is initialized here, so that when 
	 grabImage() is called later, the snap can be quickly changed to 
	 an image.
	
	 The object is initialized by taking a snap, which
	 may be an actual picture or be 'empty'.
	
	 An 'empty' snap is a Buffer object with no video information,
	 as detected by examining its component VideoFormat data. 
	
	 An 'empty' snap is caused by the delay in the player, which 
	 although in its started state may still take several seconds to 
	 start capturing.
	
	 The dimensions of the snap are used to calculate the scale
	 factor from the original image size to size*size.
	 * @return state
	 */
	private boolean hasBufferToImage(){
		Buffer buf = fg.grabFrame();     // take a snap
		
		if (buf == null) {
			System.out.println("No grabbed frame");
			return false;
		}
		
		// there is a buffer, but check if it's empty or not
		VideoFormat vf = (VideoFormat) buf.getFormat();
		if (vf == null) {
			System.out.println("No video format");
			return false;
		}
		System.out.println("Video format: " + vf);
		
		// initialize bufferToImage with the video format info.
		bufferToImage = new BufferToImage(vf);
		return true;
	} 


	/**
	 * Capture an image/frame.
	 * The frame is converted from Buffer object to Image,
	 * and finally to BufferedImage.
	 * @return a BufferedImage object contains the frame
	 */
	synchronized public BufferedImage grabImage(){
		if (closedDevice)
			return null;
		
		// grab the current frame as a buffer object
		Buffer buf = fg.grabFrame();
		if (buf == null) {
			System.out.println("No grabbed buffer");
			return null;
		}

		// convert buffer to image
		Image im = bufferToImage.createImage(buf);
		if (im == null) {
			System.out.println("No grabbed image");
			return null;
		}
		
		return makeBIM(im);
	} 


	/**
	 * Make a BufferedImage copy of im, assuming no alpha channel.
	 * Resize it to fit into size dimensions.
	 * @param im Image object
	 * @return a BufferedImage object
	 */
	private BufferedImage makeBIM(Image im){
		BufferedImage copy = new BufferedImage(im.getWidth(null), im.getHeight(null), BufferedImage.TYPE_INT_RGB);
	    
		Graphics2D g2d = copy.createGraphics();
		g2d.drawImage(im,0,0,null);
		g2d.dispose();
		
		return copy;
	}

	
	/**
	 * close() and grabImage() are synchronized so that it's not
	 * possible to close down the player while a frame is being
	 * snapped.
	 */
	synchronized public void close(){  
		p.close();  
		closedDevice = true;
	} 

	
	/**
	 * is closed the device?
	 * @return true if device is closed
	 */
	public boolean isClosed(){  
		return closedDevice;  
	}


	/**
	 * respond to events
	 * @see javax.media.ControllerListener#controllerUpdate(javax.media.ControllerEvent)
	 */
	public void controllerUpdate(ControllerEvent evt){
		if (evt instanceof StartEvent) {   // the player has started
			synchronized (waitSync) {
				stateTransitionOK = true;
				waitSync.notifyAll();
			}
		}
		else if (evt instanceof ResourceUnavailableEvent) {  
			synchronized (waitSync) {  // there was a problem getting a player resource
				stateTransitionOK = false;
				waitSync.notifyAll();
			}
		}
	}


} 
